SystemOrganization addCategory: #'Zinc-AWS'!

Object subclass: #ZnAWSS3Client
	instanceVariableNames: 'accessKeyId secretAccessKey checkIntegrity httpClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-AWS'!

!ZnAWSS3Client commentStamp: 'SvenVanCaekenberghe 9/17/2011 14:06' prior: 0!
I am ZnAWSS3Client, a basic AWS S3 Client (http://en.wikipedia.org/wiki/Amazon_S3).

| client |
(client := ZnAWSS3Client new)
	accessKeyId: '2ZGSSBGBHQGJ9VV5N441';
	secretAccessKey: 'OtxrzxIsfpFjA7SwPzILwy8Bw21TLhquhboDYROV';
	checkIntegrity: true.
client buckets.
client keysIn: 'my-bucket'.
client keysIn: 'my-bucket' query: (Dictionary with: 'prefix'->'my-').
client at: 'my-bucket' -> 'my-key'.
client at: 'my-bucket' -> 'my-key' put: (ZnEntity with: '0123456789').
client at: 'my-bucket' -> 'my-key' put: (ZnEntity with: 'Smalltalk rules S3!!') headers: (Dictionary with: 'x-amz-acl'->'public-read').
client remove: 'my-bucket' -> 'my-key'.
client removeBucket: 'my-bucket'.
client close.

I depend on Zinc HTTP Components and XML Support.!

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:07'!
accessKeyId
	"The AWS Security Credentials' Access Keys' Access Key ID used by me"
	
	^ accessKeyId! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:08'!
accessKeyId: string
	"Set my AWS Security Credentials' Access Keys' Access Key ID"

	accessKeyId := string! !

!ZnAWSS3Client methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/17/2011 13:39'!
addAuthorization
	| authorization |
	authorization := self authorizationFor: self httpClient request.
	self httpClient 
		headerAt: 'Authorization' 
		put: (String streamContents: [ :stream |
				stream nextPutAll: 'AWS '; nextPutAll: self accessKeyId; nextPut: $:; nextPutAll: authorization ]).


! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/23/2011 16:16'!
addBucket: bucket
	"Create a new AWS S3 bucket"

	self endPoint: self standardEndPoint.
	self httpClient url: bucket.
	^ self executeRequest: #put
! !

!ZnAWSS3Client methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/17/2011 13:40'!
addContentHash
	| md5 |
	self httpClient request hasEntity ifFalse: [ ^ self ].
	md5 := self md5: self httpClient request contents.
	self httpClient headerAt: 'Content-MD5' put: (ZnUtils encodeBase64: md5)


! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/23/2011 16:17'!
at: bucketKeyAssociation
	"Access an AWS S3 object in a bucket,
	identified by a bucket->key association"
	
	self endPoint: (self endPointForBucket: bucketKeyAssociation key).
	self httpClient url: bucketKeyAssociation value.
	^ self executeRequest: #get! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:16'!
at: bucketKeyAssociation put: object
	"Set an AWS S3 object in a bucket to object,
	assumed to be ZnEntity compatible,
	identified by a bucket->key association"

	^ self at: bucketKeyAssociation put: object headers: nil! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/23/2011 16:19'!
at: bucketKeyAssociation put: object headers: headers
	"Set an AWS S3 object in a bucket to object,
	assumed to be ZnEntity compatible,
	identified by a bucket->key association
	adding headers to the request"
	
	self endPoint: (self endPointForBucket: bucketKeyAssociation key).
	self httpClient url: bucketKeyAssociation value.
	self httpClient entity: object.
	headers notNil 
		ifTrue: [ self httpClient headerAddAll: headers ].
	^ self executeRequest: #put! !

!ZnAWSS3Client methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 3/21/2011 20:29'!
authorizationFor: request
	| canonicalString hmac |
	request headers at: 'Date' put: ZnUtils httpDate.
	canonicalString := self canonicalStringFor: request.
	hmac := self hmacSha1: canonicalString.
	^ ZnUtils encodeBase64: hmac ! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/23/2011 19:27'!
bucketFromEndPoint: host
	| position |
	position := host indexOfSubCollection: self standardEndPoint.
	^ position < 3
		ifTrue: [ '' ]
		ifFalse: [ '/', (host copyFrom: 1 to: position - 2) ]! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/23/2011 15:28'!
buckets
	"Return an array of all  AWS S3 bucket name strings for the account"
	
	| response |
	self endPoint: self standardEndPoint.
	self httpClient url: '/'.
	response := self executeRequest: #get.
	^ (response isSuccess and: [ response contentType = ZnMimeType applicationXml ])
		ifTrue: [
			self processBucketsXml: response contents ]
		ifFalse: [
			self error: 'Could not list buckets' ]! !

!ZnAWSS3Client methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 11/23/2011 16:11'!
canonicalStringFor: request
	^ String streamContents: [ :stream |
		stream nextPutAll: request method; lf.
		(request headers includesKey: 'Content-MD5')
			ifTrue: [ stream nextPutAll: (request headers at: 'Content-MD5') ].
		stream lf.
		request headers hasContentType ifTrue: [
			stream print: request headers contentType ].
		stream lf.
		(request headers includesKey: 'Date')
			ifTrue: [ stream nextPutAll: (request headers at: 'Date') ].
		stream lf.
		self printAmzHeaders: request headers to: stream.
		stream 
			nextPutAll: (self bucketFromEndPoint: request uri host); 
			nextPutAll: request uri pathPrintString ]! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:09'!
checkIntegrity
	"Do I check the integrety of get/put requests using content hashes ?"
	
	checkIntegrity isNil ifTrue: [ checkIntegrity := false ].
	^ checkIntegrity! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:09'!
checkIntegrity: boolean
	"Set whether I should check the integrety of get/put requests using content hashes"

	checkIntegrity := boolean! !

!ZnAWSS3Client methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/17/2011 11:22'!
close
	"Close the network resources we are using"
	
	httpClient notNil 
		ifTrue: [ httpClient close ]! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/23/2011 19:49'!
eTag
	^ (self httpClient response 
			headers at: 'ETag' ifAbsent: [ ^ nil ]) withoutQuoting! !

!ZnAWSS3Client methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/23/2011 16:25'!
endPoint: host
	"Initialize me to talk to the endpoint named host.
	Create a new HTTP client if needed, else reuse the existing one."
	
	httpClient ifNil: [ (httpClient := ZnClient new) http ]. 
	httpClient host: host! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/23/2011 15:28'!
endPointForBucket: bucketKey
	^ String streamContents: [ :stream |
		stream 
			nextPutAll: bucketKey;
			nextPut: $.; 
			nextPutAll: self standardEndPoint ]! !

!ZnAWSS3Client methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 11/23/2011 19:47'!
enforceContentHash
	| etag md5 request  |
	request := self httpClient request.
	request hasEntity ifFalse: [ ^ self ].
	(etag := self eTag) ifNil: [ ^ self ].
	md5 := self md5: request contents.
	etag = md5 hex
		ifFalse: [ self error: 'Content MD5 hash and ETag do not match' ]! !

!ZnAWSS3Client methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/17/2011 13:51'!
executeRequest: method
	self httpClient method: method asUppercase.
	self prepareRequest.
	self httpClient perform: method.
	self postProcessResponse.
	^ self httpClient response

! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 22:42'!
hmacSha1: string
	^ (HMAC on: SHA1 new)
		key: self secretAccessKey asByteArray;
		digestMessage: string asByteArray! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2011 11:23'!
httpClient
	httpClient isNil ifTrue: [ self initializeHttpClient ].
	^ httpClient! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/23/2011 15:28'!
initializeHttpClient
	self endPoint: self standardEndPoint ! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/5/2011 20:56'!
keysIn: bucket
	"Return a collection of ZnAWSS3Key objects describing the keys in bucket"
	
	^ self keysIn: bucket query: nil! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/23/2011 15:41'!
keysIn: bucket query: query
	"Return a collection of ZnAWSS3Key objects describing the keys in bucket
	using optional URI query fields"
	
	| response |
	self endPoint: (self endPointForBucket: bucket).
	self httpClient url: '/'.
	query notNil ifTrue: [ self httpClient queryAddAll: query ].
	response := self executeRequest: #get.
	^ (response isSuccess and: [ response contentType = ZnMimeType applicationXml ])
		ifTrue: [ 
			self processKeysXml: response contents for: bucket ]
		ifFalse: [
			self error: 'Could not list keys' ]! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 11:30'!
md5: object
	^ MD5 new hashMessage: object! !

!ZnAWSS3Client methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/17/2011 13:41'!
postProcessResponse
	self checkIntegrity 
		ifTrue: [ self enforceContentHash ]! !

!ZnAWSS3Client methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/17/2011 13:37'!
prepareRequest
	self checkIntegrity 
		ifTrue: [ self addContentHash ].
	self addAuthorization
	

! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 19:42'!
printAmzHeaders: headers to: stream
	| amzHeaders |
	amzHeaders := OrderedCollection new.
	headers headersDo: [ :key :value |
		(key beginsWith: 'X-Amz')
			ifTrue: [ 
				amzHeaders add: key asLowercase -> value ] ].
	amzHeaders sorted do: [ :each |
		stream 
			nextPutAll: each key; 
			nextPut: $:; 
			nextPutAll: each value; 
			lf ]! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 20:54'!
processBucketsXml: string
	| dom |
	dom := XMLDOMParser parse: string.
	^ ((dom allElementsNamed: 'Bucket') 
			collect: [ :each | 
				each contentStringAt: 'Name']) asArray! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 20:55'!
processKeysXml: string for: bucket
	| dom |
	dom := XMLDOMParser parse: string.
	^ ((dom allElementsNamed: 'Contents') 
			collect: [ :each | 
				(ZnAWSS3Key fromXml: each) 
					bucket: bucket; 
					yourself ]) asArray! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/23/2011 16:21'!
remove: bucketKeyAssociation
	"Remove an  AWS S3 object in a bucket,
	identified by a bucket->key association"

	self endPoint: (self endPointForBucket: bucketKeyAssociation key).
	self httpClient url: bucketKeyAssociation value.
	^ self executeRequest: #delete! !

!ZnAWSS3Client methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/23/2011 16:16'!
removeBucket: bucket
	"Remove an AWS S3 bucket"
	
	self endPoint: self standardEndPoint.
	self httpClient url: bucket.
	^ self executeRequest: #delete
! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:07'!
secretAccessKey
	"The AWS Security Credentials' Access Keys' Secret Access Key used by me"
	
	^ secretAccessKey! !

!ZnAWSS3Client methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 20:07'!
secretAccessKey: string
	"Set my AWS Security Credentials' Access Keys' Secret Access Key"

	secretAccessKey := string! !

!ZnAWSS3Client methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/23/2011 15:28'!
standardEndPoint
	^ 's3.amazonaws.com'! !

Object subclass: #ZnAWSS3Key
	instanceVariableNames: 'key size etag bucket xml bucketKeyAssociation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-AWS'!

!ZnAWSS3Key commentStamp: 'SvenVanCaekenberghe 1/5/2011 16:11' prior: 0!
I am ZnAWSS3Key.

I hold each result from listing the keys in an AWS S3 bucket.

Use #value to convert me to what ZnAWSS3Client>>#at: 
and ZnAWSS3Client>>#at:put: expect (a bucket->key association).!

!ZnAWSS3Key class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/8/2011 17:16'!
bucket: bucket key: key
	^ (self new)
		bucket: bucket;
		key: key;
		yourself! !

!ZnAWSS3Key class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/8/2011 17:20'!
fromAssociation: association
	^ self bucket: association key key: association value! !

!ZnAWSS3Key class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/5/2011 15:21'!
fromXml: dom
	^ self new fromXml: dom; yourself! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/8/2011 17:14'!
asResource
	^ String streamContents: [ :stream |
		stream 
			nextPut: $/; 
			nextPutAll: bucketKeyAssociation key; 
			nextPut: $/; 
			nextPutAll: bucketKeyAssociation value ]! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/8/2011 17:19'!
asUrl
	^ (ZnUrl new)
		scheme: #http;
		host: 's3.amazonaws.com';
		parseFrom: self asResource;
		yourself! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
bucket
	^ bucket! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
bucket: anObject
	bucket := anObject! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:37'!
bucketKeyAssociation
	bucketKeyAssociation isNil ifTrue: [ 
		bucketKeyAssociation := bucket -> key ].
	^ bucketKeyAssociation! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
etag
	^ etag! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
etag: anObject
	etag := anObject! !

!ZnAWSS3Key methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/5/2011 15:31'!
fromXml: dom
	self 
		key: (dom contentStringAt: 'Key');
		etag: (dom contentStringAt: 'ETag') withoutQuoting; 
		size: (Integer readFrom: (dom contentStringAt: 'Size') ifFail: [ -1 ]); 
		xml: dom
! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
key
	^ key! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
key: anObject
	key := anObject! !

!ZnAWSS3Key methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:37'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	stream print: self bucketKeyAssociation.
	stream nextPut: $)! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:25'!
size
	^ size! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
size: anObject
	size := anObject! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:43'!
value
	^ self bucketKeyAssociation! !

!ZnAWSS3Key methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
xml
	^ xml! !

!ZnAWSS3Key methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/5/2011 15:20'!
xml: anObject
	xml := anObject! !
